@@ -830,10 +830,10 @@ long long numa_node_size64(int node, long long *freep)
 
 make_internal_alias(numa_node_size64);
 
-long long numa_node_size(int node, long long *freep)
+long numa_node_size(int node, long *freep)
 {
 	long long f2;
-	long long sz = numa_node_size64_int(node, &f2);
+	long sz = numa_node_size64_int(node, &f2);
 	if (freep)
 		*freep = f2;
 	return sz;
@@ -1828,7 +1828,14 @@ static struct bitmask *__numa_preferred(void)
 
 int numa_preferred(void)
 {
-	return numa_find_first(__numa_preferred());
+	int first_node = 0;
+	struct bitmask *bmp;
+
+	bmp = __numa_preferred();
+	first_node = numa_find_first(bmp);
+	numa_bitmask_free(bmp);
+	
+	return first_node;
 }
 
 static void __numa_set_preferred(struct bitmask *bmp)
@@ -1854,9 +1861,14 @@ int numa_has_preferred_many(void)
 
 void numa_set_preferred_many(struct bitmask *bitmask)
 {
+	int first_node = 0;
+
 	if (!has_preferred_many) {
-		numa_error("Unable to handle MANY preferred nodes. Falling back to first node\n");
-		__numa_set_preferred(bitmask);
+		numa_warn(W_nodeparse,
+			"Unable to handle MANY preferred nodes. Falling back to first node\n");
+		first_node = numa_find_first(bitmask);
+		numa_set_preferred(first_node);
+		return;
 	}
 	setpol(MPOL_PREFERRED_MANY, bitmask);
 }
